/** @file ioctl.c
*
* @author Sylvain Nahas
*
* This module implements the apci3001_do_ioctl function
*/

/** @par LICENCE
* @verbatim
 Copyright (C) 2004,2005  ADDI-DATA GmbH for the source code and the documentation.
        
        ADDI-DATA GmbH
        Dieselstrasse 3 
        D-77833 Ottersweier
        Tel: +19(0)7223/9493-0
        Fax: +49(0)7223/9493-92
        http://www.addi-data-com
        info@addi-data.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. 
        
    You shoud find the complete GPL in the COPYING file accompanying 
    this source code. 
* @endverbatim
*/

#include <linux/version.h>
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,19)
    #include <linux/config.h>
#else
	#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,33)
		#include <linux/autoconf.h>
	#else
		#include <generated/autoconf.h>
	#endif
#endif

#include <linux/module.h>
#include <linux/fs.h>
#include <linux/pci.h>
#include <asm/io.h>
#if LINUX_VERSION_CODE < KERNEL_VERSION(3,4,0)
	#include <asm/system.h>
#endif
#include <asm/uaccess.h>
#include <asm/bitops.h>
#include <linux/sched.h>

#include <linux/interrupt.h>
#include <linux/kernel.h>
#include <linux/delay.h>

#include "apci3001-private.h"
#include "vtable.h"

EXPORT_NO_SYMBOLS;

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,27)
#define __user 
#endif
//------------------------------------------------------------------------------
/** dummy command to be called when a ioctl command is incorrect */
static int apci3001_do_dummy(struct pci_dev * pdev, unsigned int cmd, unsigned long arg)
{
	printk (KERN_WARNING "%s: %d: invalid ioctl\n",__DRIVER_NAME,_IOC_NR(cmd));
	return -EINVAL;
}
//------------------------------------------------------------------------------
/** add new ioctl handlers here */
void apci3001_init_vtable(vtable_t vtable)
{		
	apci3001_dummyze_vtable(vtable, apci3001_do_dummy);
	

	__APCI_3001_DECLARE_IOCTL_HANDLER(vtable,CMD_APCI3001_CheckAndGetPCISlotNumber,do_CMD_APCI3001_CheckAndGetPCISlotNumber);
	__APCI_3001_DECLARE_IOCTL_HANDLER(vtable,CMD_APCI3001_GetHardwareInformation,do_CMD_APCI3001_GetHardwareInformation);	
	__APCI_3001_DECLARE_IOCTL_HANDLER(vtable,CMD_APCI3001_SetBoardInformation,do_CMD_APCI3001_SetBoardInformation);
	
	/* Simple analog input */
	__APCI_3001_DECLARE_IOCTL_HANDLER(vtable,CMD_APCI3001_Read1AnalogInput,do_CMD_APCI3001_Read1AnalogInput);
	__APCI_3001_DECLARE_IOCTL_HANDLER(vtable,CMD_APCI3001_ReadMoreAnalogInputs,do_CMD_APCI3001_ReadMoreAnalogInputs);

	__APCI_3001_DECLARE_IOCTL_HANDLER(vtable,CMD_APCI3001_InitReadMoreAnalogInput,do_CMD_APCI3001_InitReadMoreAnalogInput);
	__APCI_3001_DECLARE_IOCTL_HANDLER(vtable,CMD_APCI3001_StartReadMoreAnalogInput,do_CMD_APCI3001_StartReadMoreAnalogInput);
	__APCI_3001_DECLARE_IOCTL_HANDLER(vtable,CMD_APCI3001_TestEndOfReadMoreConversion,do_CMD_APCI3001_TestEndOfReadMoreConversion);
	__APCI_3001_DECLARE_IOCTL_HANDLER(vtable,CMD_APCI3001_TestReadMoreAnalogInput,do_CMD_APCI3001_TestReadMoreAnalogInput);
	__APCI_3001_DECLARE_IOCTL_HANDLER(vtable,CMD_APCI3001_ReadMoreValues,do_CMD_APCI3001_ReadMoreValues);

//------------------------------------------------------------------------------

/* Digital I/O */
	__APCI_3001_DECLARE_IOCTL_HANDLER(vtable,CMD_APCI3001_Read4DigitalInput,do_CMD_APCI3001_Read4DigitalInput);
	__APCI_3001_DECLARE_IOCTL_HANDLER(vtable,CMD_APCI3001_SetOutputMemoryOn,do_CMD_APCI3001_SetOutputMemoryOn);
	__APCI_3001_DECLARE_IOCTL_HANDLER(vtable,CMD_APCI3001_SetOutputMemoryOff,do_CMD_APCI3001_SetOutputMemoryOff);
	__APCI_3001_DECLARE_IOCTL_HANDLER(vtable,CMD_APCI3001_Set4DigitalOutputOn,do_CMD_APCI3001_Set4DigitalOutputOn);
	__APCI_3001_DECLARE_IOCTL_HANDLER(vtable,CMD_APCI3001_Set4DigitalOutputOff,do_CMD_APCI3001_Set4DigitalOutputOff);

}

//------------------------------------------------------------------------------
int apci3001_do_ioctl(struct pci_dev * pdev,unsigned int cmd,unsigned long arg)
{

	/* boundaries check 
	 * 
	 * VTABLE_ELEMENT_NB(vtable_t) = __APCI3001_UPPER_IOCTL_CMD +1
	 * maximum index value = __APCI3001_UPPER_IOCTL_CMD = VTABLE_ELEMENT_NB(vtable_t) -1
	 * 
	 * the idea here is to favorize compilation-time 
	 * 
	 * */
	
	if (_IOC_NR(cmd) > (VTABLE_ELEMENT_NB(vtable_t)-1) )
		return apci3001_do_dummy(pdev,cmd,arg);
 
	/* call actual ioctl handler - should be safe now */
	return (apci3001_vtable[_IOC_NR(cmd)]) (pdev, cmd, arg);
}
//------------------------------------------------------------------------------
int do_CMD_APCI3001_CheckAndGetPCISlotNumber(struct pci_dev * pdev, unsigned int cmd, unsigned long arg)
{
	int retval = 0; /* return value of this call is the number of boards */
	
    /* this command returns an area of size CONFIG_APCI3001_MAX_BOARD_NBR+1 filled with the PCI_SLOT() value and as last field the major number associated to the device - this is deprecated and should not be used anymore.
	The call returns 0 when memory is too low or the number of boards  */	
   
	char * b_SlotArray = kmalloc(CONFIG_APCI3001_MAX_BOARD_NBR+1 * sizeof(char),GFP_KERNEL);
    if (!b_SlotArray)
	{
		/* bad param. Let output some information for debug */
		printk("%s: CMD_APCI3001_CheckAndGetPCISlotNumber: -EINVAL\n",__DRIVER_NAME);
		return 0;
	}
	    
	memset(b_SlotArray,0,CONFIG_APCI3001_MAX_BOARD_NBR+1 );

	printk("%s: the use of CMD_APCI3001_CheckAndGetPCISlotNumber is deprecated\n",__DRIVER_NAME);

	/* record the PCI_SLOT for each device from 0 to CONFIG_APCI3001_MAX_BOARD_NBR  */
	{	    	
		int i;
		for(i = 0; ( ( i < atomic_read(&apci3001_count) ) || ( i< CONFIG_APCI3001_MAX_BOARD_NBR ) ); i++)
		{
			
			struct pci_dev * dev =  apci3001_lookup_board_by_index(i);
			
			if (!dev) break;
			
			b_SlotArray[i] = PCI_SLOT(dev->devfn);
			
		}
    }
    b_SlotArray[CONFIG_APCI3001_MAX_BOARD_NBR] = apci3001_majornumber;

    if ( copy_to_user ( (unsigned char __user *)arg, b_SlotArray, sizeof (BYTE[CONFIG_APCI3001_MAX_BOARD_NBR+1])) )
	{
    	/* bad address. Let output some information for debug */
    	printk("%s: CMD_APCI3001_CheckAndGetPCISlotNumber: -EFAULT\n",__DRIVER_NAME);
    	kfree(b_SlotArray);
    	return 0;
	}
	/* return the smaller value between CONFIG_APCI3001_MAX_BOARD_NBR and number of board - Note: apci3001_count is assumed to to be always consistent with the system PCI devices list */
    retval = ( atomic_read(&apci3001_count) < CONFIG_APCI3001_MAX_BOARD_NBR ? atomic_read(&apci3001_count) : CONFIG_APCI3001_MAX_BOARD_NBR );

	kfree(b_SlotArray);	            	
	
	return retval;	

}



//------------------------------------------------------------------------------
/** Returns the informations of base address, IRQ to the user.
 * @deprecated: use-space doesn't need it and kernel-space has better means to get it.
 *  */
int do_CMD_APCI3001_GetHardwareInformation(struct pci_dev * pdev, unsigned int cmd, unsigned long arg)
{
	str_BaseInformations s_BaseInformations;
	s_BaseInformations.ui_BaseAddress[0] = GET_BAR0(pdev);
	s_BaseInformations.ui_BaseAddress[1] = GET_BAR1(pdev);
	s_BaseInformations.ui_BaseAddress[2] = 0;
	s_BaseInformations.ui_BaseAddress[3] = 0;
	s_BaseInformations.ui_BaseAddress[4] = 0;
	s_BaseInformations.b_Interrupt = pdev->irq;
	s_BaseInformations.b_SlotNumber = PCI_SLOT(pdev->devfn);

    if ( copy_to_user ((str_BaseInformations __user *)arg,&s_BaseInformations, sizeof(s_BaseInformations)) )
		return -EFAULT;
	
	return 0;
}

//------------------------------------------------------------------------------
int do_CMD_APCI3001_SetBoardInformation(struct pci_dev * pdev, unsigned int cmd, unsigned long arg)
{
	/* Test the number of channels */
	if ((arg < 4) || (arg > 16))
		return 3;	
	
	APCI3001_PRIVDATA(pdev)->nrAnaIn = (BYTE)arg;
	
	return 0;
}
